<?php

require_once ORATOR_BASE . '/lib/XMLTree.php';

/**
 * An object that provides a interface to a presentation file.
 *
 * $Horde: orator/lib/Presentation.php,v 1.10 2004/04/09 18:54:30 chuck Exp $
 *
 * @version $Revision: 1.10 $
 * @package Orator
 */
class Presentation extends XMLTree {

    var $path;
    var $p;

    /**
     *  Constructor for Presentation
     *  @access public
     */
    function Presentation()
    {
        $this->XMLTree();
        $this->header = "<?xml version=\"1.0\"?>\n";
    }

    /**
     * Override XMLTree::load() to just set the filename correctly.
     *
     * @param string $p The presentation to load.
     */
    function load($p)
    {
        $this->p = $p;
        $this->path = $GLOBALS['conf']['paths']['presentations'] . '/' . $p;
        return parent::load($this->path . '/presentation.xml');
    }

    function &get_tag_data ($name)
    {
        if( !($node = &$this->_get_node($this->tree, $name )) )
            return $this->__error( "get_$name: Could not find $name tag" );

        return $node["children"][0]["attribs"]["content"];
    }

    function &getTitle(){ return $this->get_tag_data( "title" ); }
    function &get_description(){ return $this->get_tag_data( "description" ); }
    function &get_location(){ return $this->get_tag_data( "location" ); }
    function &get_date(){ return $this->get_tag_data( "date" ); }

    function get_presentor_list(){
        if( !($preslist = &$this->_get_node($this->tree, "presentors" )) )
            return $this->__error( "get_presentor_list: Could not find presentor list" );

        $ret = Array();
        if( is_array( $preslist["children"] ) ){

            for( $i = 0; $i < sizeof( $preslist["children"] ); $i++ ){
                if( is_array( $preslist["children"][$i] ) &&
                    $preslist["children"][$i]["name"] == "presentor" ){

                    $ret[] = Array( "name" => $preslist["children"][$i]["attribs"]["name"],
                                    "organization" => $preslist["children"][$i]["attribs"]["organization"],
                                    "email" => $preslist["children"][$i]["attribs"]["email"] );

                }
            }
        }
        return $ret;
    }

    function get_slide_list(){
        if (!($slidelist = &$this->_get_node($this->tree, 'slides')))
            return $this->__error( 'get_slide_list: Could not find slides list' );

        $ret = Array();
        if (is_array($slidelist['children'])) {

            for ($i = 0; $i < sizeof($slidelist['children']); $i++) {
                if (is_array($slidelist['children'][$i]) &&
                    $slidelist['children'][$i]['name'] == 'slide') {

                    $ret[] = Array('src' => $slidelist['children'][$i]['attribs']['src'],
                                   'title' => $slidelist['children'][$i]['attribs']['title'],
                                   'duration' => $slidelist['children'][$i]['attribs']['duration'],
                                   'effect' => $slidelist['children'][$i]['attribs']['effect']);
                }
            }
        }
        return $ret;
    }

    function getResourceList()
    {
        if (!($resourcelist = &$this->_get_node($this->tree, 'resources'))) {
            return array();
        }

        $ret = array();
        if (is_array($resourcelist['children'])) {
            for ($i = 0; $i < count($resourcelist['children']); $i++) {
                if (is_array($resourcelist['children'][$i]) &&
                    $resourcelist['children'][$i]['name'] == 'resource') {

                    $ret[$resourcelist['children'][$i]['attribs']['src']] = array(
                        'src' => $resourcelist['children'][$i]['attribs']['src'],
                        'type' => $resourcelist['children'][$i]['attribs']['type']);
                }
            }
        }
        return $ret;
    }

    function get_font_list(){
        if( !($fontlist = &$this->_get_node($this->tree, 'fonts' )) )
            return $this->__error( 'get_font_list: Could not find fonts list' );

        $ret = Array();
        if( is_array( $fontlist['children'] ) ){

            for( $i = 0; $i < sizeof( $fontlist['children'] ); $i++ ){
                if( is_array( $fontlist['children'][$i] ) &&
                    $fontlist['children'][$i]['name'] == 'font' ){

                    $ret[$fontlist['children'][$i]['attribs']['category']] =
                        $fontlist['children'][$i]['attribs']['value'];

                }
            }
        }
        return $ret;
    }

    function get_color_list(){
        if( !($colorlist = &$this->_get_node($this->tree, 'colors' )) )
            return $this->__error( 'get_color_list: Could not find colors list' );

        $ret = Array();
        if( is_array( $colorlist['children'] ) ){

            for( $i = 0; $i < sizeof( $colorlist['children'] ); $i++ ){
                if( is_array( $colorlist['children'][$i] ) &&
                    $colorlist['children'][$i]['name'] == 'color' ){

                    $ret[$colorlist['children'][$i]['attribs']['category']] =
                        $colorlist['children'][$i]['attribs']['value'];

                }
            }
        }
        return $ret;
    }

    function get_pres_info()
    {
        static $pres;
        if (!isset($pres)) {
            $pres = array();

            // get presentation title
            $pres['title'] = $this->getTitle();
            if ($pres['title'] == '') {
                $pres['title'] == 'Untitled';
            }

            // get description
            $pres['description'] = $this->get_description();

            // get date
            $pres['date'] = $this->get_date();
            if ($pres['date'] == '') {
                $pres['date'] = strftime('%m/%d/%Y');
            }

            // get presentor info
            $pres['presentors'] = $this->get_presentor_list();
            $pres['slides'] = $this->get_slide_list();
            $pres['resources'] = $this->getResourceList();
            $pres['fonts'] = $this->get_font_list();
            $pres['colors'] = $this->get_color_list();
        }

        return $pres;
    }

}
