<?php

require_once ORATOR_BASE . '/lib/XMLTree.php';

/**
 * An object that provides a interface to a presentation file
 *
 * $Horde: orator/lib/Slide.php,v 1.20 2004/09/01 01:41:04 chuck Exp $
 *
 * @version $Revision: 1.20 $
 * @package Orator
 */
class Slide extends XMLTree {

    var $presentation;
    var $slide;

    /**
     * Attempts to return a concrete Slide instance based on $driver.
     *
     * @param string $driver   The type of Slide subclass to return.  The
     *                  code is dynamically included.
     * @param string $presentation  The
     * @param array  $params  (optional) Hash containing additional paramters to be passed to
     *                  the subclass's constructor.
     *
     * @return          The newly created concrete Slide instance, or
     *                  false on an error.
     */
    function &factory($driver, $presentation, $slide, $params = array())
    {
        $driver = basename($driver);
        include_once dirname(__FILE__) . '/Slide/' . $driver . '.php';
        $class = 'Slide_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($presentation, $slide, $params);
        } else {
            Horde::fatal(PEAR::raiseError("Unable to load the definition of $class."), __FILE__, __LINE__);
        }
    }

    /**
     * Constructor for Slide
     *
     * @access public
     */
    function Slide($presentation, $slide)
    {
        global $SLIDEHEADER;

        $this->XMLTree();
        $this->header = $SLIDEHEADER;
        $this->presentation = $presentation;
        $this->slide = $slide;
    }

    function load()
    {
        $pres = $this->presentation->get_pres_info();
        return parent::load($this->presentation->path . '/' . $pres['slides'][$this->slide]['src']);
    }

    function output()
    {
        return $this->header() . $this->present() . $this->footer();
    }

    function present($node = 0)
    {
        $html = '';
        if ($node == 0) {
            $node = &$this->tree;
        }

        for ($i = 0; $i < count($node['children']); $i++) {
            $name = $node['children'][$i]['name'];

            if ($name == 'heading') {
                $html .= $this->displayHeading($node['children'][$i]);
            } else if ($name == 'para') {
                $html .= $this->displayParagraph($node['children'][$i]);
            } else if ($name == 'list') {
                $html .= $this->displayList($node['children'][$i]);
            } else if ($name == 'image') {
                $html .= $this->displayImage($node['children'][$i]);
            } else if ($name == 'example') {
                $html .= $this->displayExample($node['children'][$i]);
            } else if ($name == 'raw') {
                $html .= $this->displayRaw($node['children'][$i]);
            } else if ($name == 'reference') {
                $html .= $this->displayReference($node['children'][$i]);
            } else if ($name == 'resource') {
                $html .= $this->displayResource($node['children'][$i]);
            } else if ($name == 'note') {
                $html .= $this->displayNote($node['children'][$i]);
            } else if ($name == 'term') {
                $html .= $this->displayTerm($node['children'][$i]);
            } else if ($name == 'sub') {
                $html .= $this->displaySub($node['children'][$i]);
            } else if ($name == 'sup') {
                $html .= $this->displaySup($node['children'][$i]);
            } else if ($name == '__content__') {
                $html .= $this->displayContent($node['children'][$i]);
            }
        }

        return $html;
    }

}
