<?php

require_once ORATOR_BASE . '/lib/Slide.php';

/**
 * An object that provides an HTML interface to a presentation file.
 *
 * $Horde: orator/lib/Slide/html.php,v 1.18 2004/10/19 15:22:13 mdjukic Exp $
 *
 * @version $Revision: 1.18 $
 * @package Orator
 */
class Slide_html extends Slide {

    /**
     * Constructor for Slide_html.
     * @access public
     */
    function Slide_html($presentation, $slide)
    {
        parent::Slide($presentation, $slide);
    }

    function displayContent(&$node)
    {
        return $node['attribs']['content'];
    }

    function displayParagraph(&$node)
    {
        return '<p>' . $this->present($node) . '</p>';
    }

    function displayHeading(&$node)
    {
        $html = '';
        if (!empty($node['attribs']['class'])) {
            $html .= '<div align="center" class="' . $node['attribs']['class'] . '">';
        } else {
            $html .= '<div align="center" class="osHeading">';
        }
        $html .= $this->present($node);
        return $html . '</div>';
    }

    function displayList(&$node)
    {
        $sz = sizeof($node['children']);
        $html = '';

        if (!empty($node['attribs']['type']) && $node['attribs']['type'] == 'numbered') {
            $ltype = 'ol';
        } else {
            $ltype = 'ul';
        }

        $html .= "<$ltype>\n";
        for ($i = 0; $i < $sz; $i++) {
            if ($node['children'][$i]['name'] == 'note') {
                $html .= $this->displayNote($node['children'][$i]);
            } else if ($node['children'][$i]['name'] == 'item') {
                $html .= '<li>';
                $html .= $this->present($node['children'][$i]);
                $html .= '</li>';
            } else if ($node['children'][$i]['name'] == 'list') {
                $html .= '<li>';
                $html .= $this->displayList($node['children'][$i]);
                $html .= '</li>';
            }
        }

        return $html . "</$ltype><br />\n";
    }

    function displayImage(&$node)
    {
        return "<img src='" . Horde::applicationUrl('resource.php?p=' . $GLOBALS['p'] . '&r=' . $node['attribs']['src']) . "' alt='' border='0' /><br />\n";
    }

    function displayExample(&$node)
    {
        $example = $this->_getContents($node);
        switch ($node['attribs']['type']) {
        case 'php':
            $html = Util::bufferOutput('highlight_string', $example);
            break;

        default:
            $html = nl2br(htmlspecialchars($example));
            break;
        }
        return $html . "<br />\n";
    }

    function displayRaw(&$node)
    {
        return Util::bufferOutput('eval', '?>' . $this->_getContents($node)) . "<br />\n";
    }

    function displayReference(&$node)
    {
        if (@empty($node['attribs']['label'])) {
            return '<a target="_blank" href="' . $node['attribs']['src'] . '">' . $node['attribs']['src'] . '</a>';
        }
        return '<a target="_blank" href="' . $node['attribs']['src'] . '">' . $node['attribs']['label'] . '</a>';
    }

    function displayResource(&$node)
    {
        return Horde::link(Horde::applicationUrl('resource.php?p=' . $GLOBALS['p'] . '&r=' . $node['attribs']['src']),
                           $node['attribs']['label']) . $node['attribs']['label'] . '</a>';
    }

    function displayNote(&$node)
    {
        // Notes not displayed in presentation slides; see static.php
    }

    function displayTerm(&$node)
    {
        return '<u>' . $this->present($node) . '</u>';
    }

    function displaySub(&$node)
    {
        return '<sub>' . $this->present($node) . '</sub>';
    }

    function displaySup(&$node)
    {
        return '<sup>' . $this->present($node) . '</sup>';
    }

    function getPrevious($title)
    {
        return Horde::applicationUrl('slide.php?p=' . $this->presentation->p . '&s=' . ($this->slide - 1));
    }

    function getCurrent($title)
    {
        return Horde::applicationUrl('slide.php?p=' . $this->presentation->p . '&s=' . ($this->slide));
    }

    function getNext($title)
    {
        return Horde::applicationUrl('slide.php?p=' . $this->presentation->p . '&s=' . ($this->slide + 1));
    }

    function backImage()
    {
        return Horde::img('back.png');
    }

    function forwardImage()
    {
        return Horde::img('forward.png');
    }

    function header()
    {
        global $registry, $refresh_time, $refresh_url;

        $pres = $this->presentation->get_pres_info();
        $title = $pres['slides'][$this->slide]['title'];
        $max = count($pres['slides']);
        if ($this->slide != $max - 1) {
            $nt = $pres['slides'][$this->slide + 1]['title'];
            $refresh_time = $pres['slides'][$this->slide]['duration'];
            $refresh_url = $this->getNext($pres['slides'][$this->slide + 1]['src']);
        } else {
            $nt = '';
        }
        if ($this->slide != 0) {
            $pt = $pres['slides'][$this->slide - 1]['title'];
        } else {
            $pt = '';
        }

        $html = '';
        $html .= Util::bufferOutput('require', ORATOR_TEMPLATES . '/common-header.inc');
        $html .= "<table class='oratorHeader' border='0' cellspacing='0' cellpadding='1' width='100%'><tr>\n";
        if ($this->slide != 0) {
            $url = $this->getPrevious($pres['slides'][$this->slide - 1]['src']);
            $html .= "<td width='1%' align='left' class='oratorHeaderBack' valign='center' nowrap='nowrap'>";
            $html .= Horde::link($url, sprintf(_("Go to: %s"), $pt), null, null, null, null, 'P') . $this->backImage() . '</a>';
            $html .= '</td><td style="cursor: hand;" onclick="location.href=\'' . $url . '\';" align="left" class="oratorHeaderBack" width="24%">' . $this->getTitleText($pt, 1);
        } else {
            $html .= "<td width='25%' align='left' class='oratorHeaderBack' valign='center' nowrap='nowrap'>&nbsp;";
        }
        $html .= "</td>\n";
        $url = $this->getCurrent($pres['slides'][$this->slide]['src']);
        $html .= "<td width='50%' align='center' style='cursor: hand;' class='oratorHeaderBack' valign='center' nowrap='nowrap' onclick='location.href=\"" . $url . "\";'>";
        $html .= $this->getTitleText($title);
        $html .= "</td>\n";
        if ($this->slide != ($max - 1)) {
            $url = $this->getNext($pres['slides'][$this->slide + 1]['src']);
            $html .= "<td style='cursor: hand;' onclick='location.href=\"" . $url . "\";' width='24%' align='right' class='oratorHeaderBack' valign='center' nowrap='nowrap'>";
            $html .= $this->getTitleText($nt, 1);
            $html .= '</td><td width="1%" align="right">' . Horde::link($url, sprintf(_("Go to: %s"), $nt), null, null, null, null, 'N')
                . $this->forwardImage() . '</a>';
        } else {
            $html .= "<td width='25%' align='right' class='oratorHeaderBack' valign='center' nowrap='nowrap'>&nbsp;";
        }
        $html .= "</td>\n";
        $html .= "</tr></table>\n";
        $html .= "<br /><br /><div align='center'>\n";
        $html .= "<table border='0' width='90%'><tr><td align='left'>";

        return $html;
    }

    function footer()
    {
        global $registry;

        $html = '';
        $html .= "</td></tr></table>";
        $html .= "</div>\n";
        $html .= Util::bufferOutput('require', $registry->get('templates', 'horde') . '/common-footer.inc');

        return $html;
    }

    function getTitleText($text, $highlight = 0)
    {
        $html = '<table border="0" cellspacing="0" cellpadding="0"><tr><td nowrap="nowrap">';

        if ($highlight) {
            $class = 'oratorTitleHigh';
        } else {
            $class = 'oratorTitle';
        }
        $html .= "<div class='$class'>$text</div>";

        $html .= '</td></tr></table>';
        return $html;
    }

}
