<?php

require_once ORATOR_BASE . '/lib/Slide.php';

/**
 * An object that provides a interface to a presentation file
 *
 * $Horde: orator/lib/Slide/text.php,v 1.6 2004/04/07 17:43:43 chuck Exp $
 *
 * @version $Revision: 1.6 $
 * @package Orator
 */
class Slide_text extends Slide {

    /**
     * Constructor for Slide_text.
     * @access public
     */
    function Slide_text($presentation, $slide)
    {
        parent::Slide($presentation, $slide);
    }

    function displayContent(&$node)
    {
        return wordwrap($node['attribs']['content'], 80);
    }

    function displayParagraph(&$node)
    {
        return $this->present($node) . "\n\n";
    }

    function displayHeading(&$node)
    {
        return $this->present($node) . "\n\n";
    }

    function displayList(&$node, $indent = '  ')
    {
        $sz = sizeof($node['children']);
        $text = '';

        for ($i = 0; $i < $sz; $i++) {
            if ($node['children'][$i]['name'] == 'note') {
                $text .= $this->displayNote($node['children'][$i]);
            } else if ($node['children'][$i]['name'] == 'item') {
                if (!empty($node['attribs']['type']) && $node['attribs']['type'] == 'numbered' ){
                    $text .= $indent . $i;
                } else {
                    $text .= $indent . '*';
                }
                $text .= $this->present($node['children'][$i]);
                $text .= "\n";
            } else if ($node['children'][$i]['name'] == 'list') {
                $text .= "\n";
                $text .= $this->displayList($node['children'][$i], $indent . '  ');
            }
        }

        return $text . "\n\n";
    }

    function displayImage(&$node)
    {
        copyFile($this->presentation->path . '/' . $node['attribs']['src'], $node['attribs']['src']);
        return '[Included image ' . $node['attribs']['src'] . ']';
    }

    function displayExample(&$node)
    {
        return $this->_getContents($node) . "\n\n";
    }

    function displayRaw(&$node)
    {
        return Util::bufferOutput('eval', '?>' . $this->_getContents($node)) . "\n\n";
    }

    function displayReference(&$node)
    {
        return $node['attribs']['src'];
    }

    function displayResource(&$node)
    {
        copyFile($this->presentation->path . '/' . $node['attribs']['src'], $node['attribs']['src']);
        return '[See included file ' . $node['attribs']['label'] . ' (' . $node['attribs']['src'] . ')]';
    }

    function displayNote(&$node)
    {
        return $this->present($node) . "\n\n";
    }

    function displayTerm(&$node)
    {
        return "_" . $this->present($node) . "_";
    }

    function displaySub(&$node)
    {
        return "(" . $this->present($node) . ")";
    }

    function displaySup(&$node)
    {
        return "^" . $this->present($node);
    }

    function header()
    {
        $pres = $this->presentation->get_pres_info();
        $title = $pres['slides'][$this->slide]['title'];

        return "\n\n-----------------\n\n" . $this->slide . '. ' . $title . "\n\n";
    }

    function footer()
    {
        return '';
    }

    function getTitleText($text, $highlight = 0)
    {
        return $text;
    }

}
