<?php
/**
 * $Horde: orator/lib/XMLTree.php,v 1.7 2003/08/16 11:37:36 jan Exp $
 *
 * $Log: XMLTree.php,v $
 * Revision 1.7  2003/08/16 11:37:36  jan
 * Use absolute path names.
 *
 * Revision 1.6  2002/06/12 05:59:38  chuck
 * phpdoc/commenting fixes.
 *
 * Revision 1.5  2001/11/04 04:24:18  chuck
 * the presentation listing can be a lot prettier, but it works.
 * also added nicer prev/next icons.
 *
 * Revision 1.4  2001/10/30 22:04:00  chuck
 * multiple presentation support. no list yet; that'll come once the slides are converted.
 *
 * Revision 1.3  2001/10/29 16:21:07  chuck
 * fix missing CDATA tags in example slides, and make php
 * source examples, references, and raw tags work.
 *
 * Revision 1.2  2001/10/29 00:55:51  chuck
 * style and fixes, formatting, license, and kill some very redundant code
 *
 * Revision 1.1.1.1  2001/10/04 19:22:45  jon
 * Initial import of a modified version of Steve Bower's Orator presentation
 * system.
 */

require_once ORATOR_BASE . '/lib/error_handler.inc.php';

// serviceable constants
$__xmltree_blocksize = 4096;

// internal variables
$__xmltree_root = 0;
$__xmltree_curnode = 0;

function __xmltree_startElement( $parser, $name, $attribs ){

    if( !is_array($GLOBALS["__xmltree_root"]) ){
      $GLOBALS["__xmltree_root"] = Array( "name" => $name, 
                     "attribs" => $attribs,
                     "parent" => 0,
                     "children" => Array() );
      $GLOBALS["__xmltree_curnode"] = &$GLOBALS["__xmltree_root"];
    } else {
      $newnode = Array( "name" => $name, 
                        "attribs" => $attribs,
                        "parent" => &$GLOBALS["__xmltree_curnode"],
                        "children" => Array() );
      $GLOBALS["__xmltree_curnode"]["children"][] = &$newnode;
      $GLOBALS["__xmltree_curnode"] = &$newnode;
    }

}

function __xmltree_endElement( $parser, $name ){
  
    if( is_array($GLOBALS["__xmltree_curnode"]["parent"]) ){
      $tmpnode = &$GLOBALS["__xmltree_curnode"];
      $GLOBALS["__xmltree_curnode"] = &$GLOBALS["__xmltree_curnode"]["parent"];
      unset( $tmpnode["parent"] );
    }
}

function __xmltree_cdata($parser, $data) {
    /*
    $data = trim($data);
    if( $data[0] == "\n" )
      $data = substr( $data, 1 );
    if( $data[sizeof($data)-1] == "\n" )
      $data = substr( $data, 0, -3 );
    $data = trim($data);
    */

    if( $data != "" ){
       $newnode = Array( "name" => "__content__",
                         "attribs" => Array( "content" => $data ),
                         //"parent" => &$GLOBALS["__xmltree_curnode"],
                         "parent" => NULL,
                         "children" => Array() );
       $GLOBALS["__xmltree_curnode"]["children"][] = &$newnode;
    }
}

function bprint( &$buf, $str ){
    if (is_resource( $buf ) )
        fwrite( $buf, $str  );
    else
        $buf .= $str;
}

/**
 * An object that can parse xml docs into a treelike structure
 *
 * This class parses XML into a treelike structure. It should
 * probably be removed and replaced with and the xmllib parser in PHP. As
 * of its original writing the xmllib parser in php was not finished
 *
 * @version $Revision: 1.7 $
 * @package horde
 */
class XMLTree extends error_handler {
    /*! @var tree     array - an array containing the xml tree
     *  @var header   string - a string that will be automatically prepended
     *                         to the outputed xml doc
     *  @var footer   string - a string that will be automatically appended
     *                         to the outputed xml doc
     *  @var filename string - the filename of this xml document
     */  

    /**
     *  An array containing the xml tree
     *
     *  @var array
     */
    var   $tree;

    /**
     *  A string that will be automatically prepended to the outputed xml doc
     *
     *  @var string
     */
    var   $header = "<?xml version=\"1.0\"?>\n";

    /**
     *  A string that will be automatically appended to the outputed xml doc
     *
     *  @var string
     */
    var   $footer = "";

    /**
     *  The filename of this xml document
     *
     *  @var string
     */
    var   $filename = ""; 

    /**
     *  Constructor for xmltree
     *  @access public
     */
    function XMLTree(){
      $this->error_handler();
      $this->filename = "";
      $this->tree = 0;
    }

    /**
     *  load an xml file from disk
     *
     *  @param string The filename to load
     *  @return int 0 on success, -1 on failure
     *  @access public
     */
    function load($filename)
    {
        global $__xmltree_blocksize;
        
        $this->filename = $filename;
        
        $xml = xml_parser_create();
        xml_parser_set_option( $xml, XML_OPTION_CASE_FOLDING, 0 );
        xml_set_element_handler( $xml, "__xmltree_startElement", "__xmltree_endElement" );
        xml_set_character_data_handler($xml, "__xmltree_cdata");
        
        if( $fp = @fopen( $this->filename, "rb") ){
            while( $data = fread($fp, $__xmltree_blocksize ) ){
                if( !xml_parse( $xml, $data, feof( $fp ) ) ){
                    return $this->__error( sprintf( "XML error: %s at line %d",
                                                    xml_error_string( xml_get_error_code( $xml ) ),
                                                    xml_get_current_line_number( $xml ) ) );
                }
            }
        } else {
            return $this->__error( "Could not open '".$this->filename."' for reading" );
        }
        xml_parser_free($xml);
        
        $this->tree = $GLOBALS["__xmltree_root"];
        $GLOBALS["__xmltree_root"] = 0;
        
        return 0;
    }

    /**
     *  load an xml file from a string
     *
     *  @param string The string to load from
     *  @return int 0 on success, -1 on failure
     *  @access public
     */
    function load_string( $data ){
      global $__xmltree_blocksize;

      $this->filename = "";

      $xml = xml_parser_create();
      xml_parser_set_option( $xml, XML_OPTION_CASE_FOLDING, 0 );
      xml_set_element_handler( $xml, "__xmltree_startElement", "__xmltree_endElement" );
      xml_set_character_data_handler($xml, "__xmltree_cdata");

      if( !xml_parse( $xml, $data, true  ) ){
          return $this->__error( sprintf( "XML error: %s at line %d",
                          xml_error_string( xml_get_error_code( $xml ) ),
                          xml_get_current_line_number( $xml ) ) );
      }

      xml_parser_free($xml);

      $this->tree = $GLOBALS["__xmltree_root"];
      $GLOBALS["__xmltree_root"] = 0;

      return 0;
    }

    /**
     *  Save an xml file to disk
     *
     *  @param string   The filename to save to disk as
     *  @return int 0 on success, -1 on failure
     *  @access public
     */
    function save( $filename="" ){
      global $conf;

      $tmpfile = tempnam( $conf['tempdir'], "IStmp" );


      if( $fp = @fopen( $tmpfile, "w") ){
        $this->__save( $this->tree, $fp );
      } else {
        return $this->__error( "Could not open '$tmpfile' for writing" );
      }

      fclose( $fp );

      if( $filename == "" )
        $filename = $this->filename;
 
      if( !@copy( $tmpfile, $filename ) ){
        @unlink( $tmpfile );
        return $this->__error( "Could not copy '$tmpfile' to '$filename'" );
      }

      @unlink( $tmpfile );

      return 0;
    }

    /**
     *  Save an xml file to a string
     *
     *  @param string   The filename to save to disk as
     *  @return int 0 on success, -1 on failure
     *  @access public
     */

    function save_string(&$buffer)
    {
        return $this->__save($this->tree, $buffer);
    }

    /**
     *  Internal recursive save function
     *
     *  @param array  The current node
     *  @param int    The file descriptor to write to
     *  @param int    The current depth in the xml tree (used for formatting)
     *  @return int 0 on success, -1 on failure
     *  @access private
     *  @see save()
     */
    function __save($node, &$fp, $depth = 0)
    {

      if( !is_array( $node ) )
        return;
      if( $depth == 0 ){
        bprint( $fp, $this->header );
      }

      if( $node["name"] == "__content__" ){
        for( $i=0; $i < $depth; $i++ )
          bprint( $fp, "  " );
        bprint( $fp, $node["attribs"]["content"]."\n" );
        return 0;
      }

      for( $i=0; $i < $depth; $i++ )
        bprint( $fp, "  " );

      bprint( $fp, "<".$node["name"] );
        
      while( list( $key, $val ) = each( $node["attribs"] ) ){
        bprint( $fp, " $key=\"$val\"" );
      }

      if( sizeof( $node["children"] ) == 0 ){
        bprint( $fp, " />\n" );
      } else {
        bprint( $fp, ">\n" );
        for( $i = 0; $i < sizeof( $node["children"] ); $i++ )
          $this->__save($node["children"][$i], $fp, $depth + 1 );
        for( $i=0; $i < $depth; $i++ )
          bprint( $fp, "  " );
        bprint( $fp, "</".$node["name"].">\n" );
      }

      if( $depth == 0 ){
        bprint( $fp, $this->footer );
      }

      return 0;
    }

    /**
     *  Returns a reference to a node named $nodename
     *
     *  @param ref      A reference to a node
     *  @return ref     A reference to the node that was found
     *  @access private
     */
    function &_get_node(&$node, $nodename)
    {
        for ($i = 0; $i < sizeof($node['children']); $i++) {
            if ($node['children'][$i]['name'] == $nodename) {
                return $node['children'][$i];
            }
        }
        return 0;
    }
    
    /**
     * Returns all content that is a child of the passed-in node,
     * concatenated.
     *
     * @param ref      A reference to a node
     * @return string  All content of that node
     */
    function _getContents(&$node)
    {
        $contents = '';
        foreach ($node['children'] as $child) {
            $contents .= $child['attribs']['content'];
            if (count($child['children']) > 0) {
                $contents .= $this->_getContents($child);
            }
        }
        
        return $contents;
    }
    
}
?>
